#!/usr/bin/env python3
import unittest
from tmdb.tmdb_api import Movie, TvShow
from tmdb.tmdb import TmdbBot
from tmdb.database import Database
from sqlalchemy import create_engine


class TestTmdbMethods(unittest.TestCase):
    ### TMDB API
    def test_search_item(self):
        movie = Movie()
        id = movie.search_title('Breakfast Club')
        self.assertEqual(id, 2108)

    def test_cast(self):
        movie = Movie()
        movie.search_title('Breakfast Club')
        self.assertEqual('Anthony Michael Hall', movie.cast[0])
        
    def test_title(self):
        movie = Movie()
        movie.search_title('Breakfast Club')
        self.assertEqual('The Breakfast Club', movie.title)
        
    def test_overview(self):
        movie = Movie()
        movie.search_title('Breakfast Club')
        description = 'Five high school students from different walks of'
        self.assertEqual(description, movie.overview[:len(description)])
        
    def test_change_language(self):
        movie = Movie()
        movie.set_language('en')
        movie.search_title('Breakfast Club')
        description = 'Five high school students from different walks of life endure a Saturday detention'
        self.assertEqual(description, movie.overview[:len(description)])

    def test_html_construction(self):
        movie = Movie()
        tmdb = TmdbBot("","" ,"" ,"" ,"" ,"" ,"" ,"" ,"" )
        movie.query_details('550')
        message = tmdb.construct_html_message(movie, overview_length = 10)
        self.assertEqual(message, """<p><a href="https://www.themoviedb.org/movie/550"><b>Fight Club</b></a></p>
                           <p>A ticking- [...]</p>
                           <p>Acting: Edward Norton, Brad Pitt, Helena Bonham Carter</p>
                           <p>Taken from www.themoviedb.org</p>""")

    def test_database_language(self):
        engine = create_engine('sqlite:///test.db', echo = True)
        db = Database(engine)
        db.set_language('@testuser:example.com', 'de')
        self.assertEqual(str(db.get_language('@testuser:example.com')), 'de')
        db.set_language('@testuser:example.com', 'en')
        self.assertEqual(str(db.get_language('@testuser:example.com')), 'en')

    def test_id_lookup(self):
        movie = Movie()
        movie.query_details('2108')
        self.assertEqual('The Breakfast Club', movie.title)

    def test_search_fails(self):
        movie = Movie()
        id = movie.search_title('Breakfast Club 2019')
        self.assertEqual(id, None)
        self.assertEqual(None, movie.title)

    def test_search_year(self):
        movie = Movie()
        id = movie.search_title('Dune')
        self.assertEqual(id, 841)
        id = movie.search_title('Dune', 2020)
        self.assertEqual(id, 438631)

    def test_split_year(self):
        tmdb = TmdbBot("","" ,"" ,"" ,"" ,"" ,"" ,"" ,"" )
        title, year = tmdb.split_title_year('Dune')
        self.assertEqual('Dune', title)
        self.assertEqual(None, year)
        title, year = tmdb.split_title_year('Dune y:2020 ')
        self.assertEqual('Dune', title)
        self.assertEqual(2020, year)
        
    def test_set_poster_size(self):
        movie = Movie()
        size = movie.set_poster_size("w500")
        self.assertEqual(size, "w500")
        self.assertEqual(movie.base_url_poster, f"{movie.base_url_images}w500")
        size = movie.set_poster_size("w666")
        self.assertEqual(size, None)

    # TV Shows   
    def test_search_tvshow(self):
        movie = TvShow()
        id = movie.search_title('The Flash')
        self.assertEqual(id, 60735)

    def test_tv_title(self):
        movie = TvShow()
        movie.search_title('The Flash')
        self.assertEqual('The Flash', movie.title)

    def test_cast(self):
        movie = TvShow()
        movie.search_title('The Flash')
        #self.assertEqual('Tom Cavanagh', movie.cast[0])
        self.assertEqual('Carlos Valdes', movie.cast[2])

if __name__ == '__main__':
    unittest.main()
